<?php

class BeryliumObject {
// master class for all objects stored in the database
	
	// all database objects have these attributes
	var $id;		// unique id
	var $flavor;		// subtype of the object; indexed
	var $created;		// creation timestamp
	var $updated;		// last update timestamp
	var $sessionid;		// last session to modify this object
	var $sitememberid;	// Sitemember id of owner/creator or 0 for Anonymous; indexed
	var $siteid;		// id of containing Site or 0 for Ubiquitous; indexed
	var $folderid;		// id of containing Folder or 0 for root (/); indexed
	var $status;		// new | posted | hidden | deleted; indexed
	var $properties;		// ^-delimited list of key=value pairs for extra properties
	// operational attributes not stored in db
	var $columns;		// mySQL column_definition for this object, to be inherited by user objects
	var $queryExtras;	// extra WHERE clauses (beginning with AND) and/or OFFSET, LIMIT, and ORDERBY clauses
	var $listsize;		// number of objects returned by selectObject() (number of objects in $objectarray[])

// constructor method
	function BeryliumObject() {
		$this->columns= "id=INT UNSIGNED AUTO_INCREMENT PRIMARY KEY^flavor=VARCHAR (255)^INDEX index_flavor=(flavor(32))^created=DATETIME^updated=DATETIME^sessionid=INT UNSIGNED^sitememberid=INT UNSIGNED^INDEX index_sitememberid=(sitememberid)^siteid=INT UNSIGNED^INDEX index_siteid=(siteid)^folderid=INT UNSIGNED^INDEX index_folderid=(folderid)^status=VARCHAR(255)^INDEX index_status=(status(16))^properties=TEXT";
		}

// table backup
	function backupDB() {
		$now= date("U");
		$filename= "/home/berylium/berylium/backups/$this->objtype-$now.sql";
		$command= "/usr/local/mysql/bin/mysqldump --opt -u $GLOBALS[dbuser] -p$GLOBALS[dbpassword] beryliumdb $this->objtype > $filename";
		berror("backupDB() will attempt to perform backup of $this->objtype table.",1);

		$error= exec("$command");

		if (file_exists("$filename")) {
			$filesize= filesize("$filename");
			if ($filesize >= 1073741824) {
				$filesize = round($filesize / 1073741824 * 100) / 100 . "g";
				} 
			elseif ($filesize >= 1048576) {
				$filesize = round($filesize / 1048576 * 100) / 100 . "m";
				} 
			elseif ($filesize >= 1024) {
				$filesize = round($filesize / 1024 * 100) / 100 . "k";
				} 
			else {
				$filesize = $filesize . "b";
				}

			$this->backupsize= $filesize;
			$this->backupname= $filename;
			$error= "Created $filesize backup file at $filename.";
			$success= 1;			
			}

		berror("backupDB() success=$success, $error",1);

		return $success;
		}


// policies
	function applyPolicy() {
		// only call once per object, k?
		if ($this->publicwhere!="" && $this->statuswhere!="") {
			berror("[bold:applyPolicy():] called but publicwhere=$this->publicwhere and/or statuswhere=$this->statuswhere, so skipping lookup.",1);
			return 1;
			}
		$dbpolicies=0;
		
		$site= $GLOBALS[site];
		$sitemember= $GLOBALS[sitemember];
		$member= $GLOBALS[member];
		$postvars= $GLOBALS[HTTP_POST_VARS];
		$getvars= $GLOBALS[HTTP_GET_VARS];

		$policy= new Policy;
		$policy->flavor= $sitemember->role;
		if ($sitemember->role=="admin") $policy->flavor="editor";
		if ($sitemember->role=="anonymous" && ( $member->id=="" || !$site->public ) ) {	
			berror("applyPolicy(): has membership, but not sitemembership and site is private, policy role is anonymous.",1);
			$policy->flavor="anonymous";
			}
		elseif ($sitemember->role=="anonymous" && $member->id!="" ) {
			berror("applyPolicy(): has membership, but not sitemembership, policy role is member.",1);
			$policy->flavor="member";
			// we should create a sitemember here for future use? or make some other distinction...
			// but for now we just insert some default values in the sitemember object
			$sitemember->role= "member";
			$sitemember->name= $member->email;
			$GLOBALS[sitemember]= $sitemember;
			}
			
		$policy->folderid= $sitemember->folderid;
		$policy->siteid= $GLOBALS[site]->id;
		$policy->sitefolderid= $GLOBALS[site]->folderid;

		berror("Looking up policies now.<blockquote>",1);

		if ($dbpolicies) {
			// lookup policy in this folder first
			$query= "SELECT * FROM policy WHERE flavor='$policy->flavor' AND folderid='".$GLOBALS[folder]->id."' AND siteid='$policy->siteid' AND status='posted' ORDER BY created ASC";
			$policy->selectObject($query);
		
			if ($policy->id=="") {
		
				// lookup policy in sitemember->folderid
				$query= "SELECT * FROM policy WHERE flavor='$policy->flavor' AND folderid='".$GLOBALS[sitemember]->folderid."' AND siteid='$policy->siteid' AND status='posted' ORDER BY created ASC";
				$policy->selectObject($query);
		
				// lookup policy in root folder
				if ($policy->id=="") {
					$query= "SELECT * FROM policy WHERE flavor='$policy->flavor' AND folderid='$policy->sitefolderid' AND status='posted' ORDER BY id ASC";
					$policy->selectObject($query);
		
					if ($policy->id=="") {
						// turn errorLevel back up
						berror("</blockquote>",1);	
						berror("applyPolicy(): No policy found to match your role ($policy->flavor) in this area. [bold:Server can not continue].",0);
						return 0;
						} 
					}
				}
			}
		else {
			// lookup and load root policy file
			$policypath= $GLOBALS['beryliumroot']."/code/policies/";
			$success=0;
			
			// root policy
			$policy->name= $policy->flavor;
			berror("getpolicy(): Going to look for root policy file at $policypath$policy->name.",1);
			$path= $policypath.$policy->name;
			if (file_exists($path)) {
				$policy->uploadPolicy($path);
				$success=1;
				}
			else {
				// turn errorLevel back up
				berror("</blockquote>",1);	
				berror("applyPolicy(): No policy found to match your role ($policy->flavor) in this area. [bold:Server can not continue].",0);
				return 0;
				}			
			}

		// turn errorLevel back up
		berror("Done with policies</blockquote>",1);

		berror("applyPolicy() called by $this->objtype id#$this->id ($this->name). [bold:Found policy $policy->flavor.]",1);

		// generate addins for select queries
		if ($policy->prequery!="") {
			berror("applyPolicy() processing preselect: ".$policy->prequery,2);
			eval($policy->prequery);
			}

		// default to very restrictive extras just in case
		if ($this->statuswhere=="") $this->statuswhere= " AND obj.status='posted' ";
		if ($this->publicwhere=="") $this->publicwhere= " AND obj.public=1 ";
		if ($this->orderby=="") $this->orderby= "ORDER BY obj.id ASC";
		if ($this->insertstatus=="") $this->insertstatus= "new";
		if ($this->insertpublic=="") $this->insertpublic= "1";
		if ($this->insertrank=="") $this->insertrank= "100";
		//if ($this->updatestatus=="") $this->updatestatus= "changed";
		//if ($this->updatepublic=="") $this->updatepublic= "";
		//if ($this->updaterank=="") $this->updaterank= "";


		// generate object addons
		if ($policy->addons!="") {
			berror("applyPolicy() processing default: ".$policy->addons,2);
			eval($policy->addons);
			}

		return 1;

		}


// database interface - insert, update, select, etc.
	function insertObject() {
		// insertObject inserts $this as a new database record
		berror("insertObject() called by ".get_class($this)." id#$this->id ($this->name). objtype=$this->objtype, siteid=".$GLOBALS[site]->id.", this->siteid=$this->siteid.",1);
		$success=0;

		// connect to database
		$session= $GLOBALS[session];
		if (!$session->dbconnection) $GLOBALS[session]->dbconnection= dbconnect();

		// apply policy
		if ($this->objtype!="session" AND !$session->setup) {
			if (!($session->canSavenew[$this->objtype]  OR $session->canSave[$this->objtype] OR $session->canSave[all])) {
				berror("insertObject(): Failed to find a policy that allows you to save new $this->objtype objects. Server must halt.",0);
				exit;
				}
			if ($session->insertstatus!="") $this->status= $session->insertstatus;
			if ($session->insertpublic!="") $this->public= $session->insertpublic;
			if ($session->insertrank!="") $this->rank= $session->insertrank;
			}

		// make sure some important fields are set
		if ($this->siteid=="") $this->siteid=$GLOBALS[site]->id;
		if ($this->folderid=="") $this->folderid=$GLOBALS[folder]->id;
		if ($this->sitememberid=="") $this->sitememberid=$GLOBALS[sitemember]->id;
		$this->sessionid= $session->id;

		// check for duplicate name in this folder
		if ($this->name!="") {
			$found= $this->checkDuplicate("name");
			if ($found) {
				// if this is owned or operated, prompt for overwrite -- NOT IMPLEMENTED!
				if (0 AND ($this->sitememberid==$GLOBALS[sitemember]->id OR $GLOBALS[sitemember]->role=="editor")) {
					$found=0;
					berror("insertObject() disallowing overwrite by owned and operated.",1);
					}
				
				// if this is in a different site, forget it unless this bit is set.
				elseif ($GLOBALS[allowoverwrite] AND $GLOBALS[sitemember]->rank=="admin") {
					berror("insertObject() allowing overwrite by admin.",1);
					$this->id= $found;
					$overwrite= $this->updateObject;
		
					return $overwrite;
					}

				else {
					berror("checkDuplicate(): A $this->objtype with the same $field already exists here. It is $this->objtype id#$found[id] with status=$found[status]. Please use the back button to change the name of the object you are trying to create or edit.",0);
					return 0;
					}
				}
			}

		// addslashes to all fields
		$this->beryliumToMySQL();



		// get columns from table description so we know what bits of this object to save
		$query= "DESCRIBE $this->objtype ";
		$result= mysql_query($query);
		$a=1;
		while ($columnarray= mysql_fetch_array($result)) {
			$thisfield= $columnarray[Field];
			$tablecolumn[$thisfield]= $a;
			$a++;
			}

		// run through the columns to build the insert query... created and updated are exceptional
		$queryvalues= "''";
		while (list($key,$val)= each($tablecolumn)) {
			if ($key=="id") {
				continue;
				}
			elseif ($key=="created" OR $key=="updated") {
				$queryvalues= $queryvalues.", now()";
				continue;
				}
			else {
				$newobject[$key]= $this->{$key};
				$queryvalues= $queryvalues.", '$newobject[$key]'";
				}
			}		

		// process insert query
		$query= "INSERT INTO $this->objtype VALUES ($queryvalues) ";
		$result= mysql_query($query);
		if (!$result) {
			berror("insertObject(): insert query failed to get a result. $query",1);
			}
		else {
			berror("insertObject(): [bold:Insert successful.] $query",1);
			$success= 1;
			}

		// update object with new id and unencode
		$this->id= mysql_insert_id();
		$this->mySQLToBerylium();

		return $success;
		}

	function updateObject() {
		// updateObject updates the database record for $this
		berror("updateObject() called by ".get_class($this)." id#$this->id ($this->name) (path=$this->path) (status=$this->status)",1);
		$success=0;
		$postvars= $GLOBALS[HTTP_POST_VARS];

		// connect to database
		$session= $GLOBALS[session];
		if (!$session->dbconnection) $GLOBALS[session]->dbconnection= dbconnect();
		$this->sessionid= $session->id;

		// apply policy
		if ($this->objtype!="session") {
			if (!($session->canSave[$this->objtype] OR $session->canSave[all])) {
				berror("updateObject(): Failed to find a policy that allows you to save $this->objtype objects. Server must halt.",0);
				exit;
				}
			if ($session->updatestatus!="") $this->status= $session->updatestatus;
			if ($session->updatepublic!="") $this->public= $session->updatepublic;
			if ($session->updaterank!="") $this->rank= $session->updaterank;
			}

		// check for duplicate name in this folder
		if ($this->siteid=="") $this->siteid=$GLOBALS[site]->id;
		if ($this->name!="") {
			$found= $this->checkDuplicate("name");
			if ($found) {
				return 0;
				}
			}

		// addslashes to all fields
		$this->beryliumToMySQL();

		// get columns from table description so we know what bits of this object we can save
		$query= "DESCRIBE $this->objtype ";
		berror("updateObject inspecting table with query=$query; ",2);
		$result= mysql_query($query);
		$a=1;
		while ($columnarray= mysql_fetch_array($result)) {
			berror("updateObject() extracting column $columnarray[Field].",2);
			$thisfield= $columnarray[Field];
			$tablecolumn[$thisfield]= $a;
			$a++;
			}

		// run through columns to build the update query
		$queryvalues= "updated= now()";
		foreach ($tablecolumn AS $key=>$val) {
			if ($key=="id" OR $key=="created" OR $key=="updated") {
				berror("updateObject() continuing because of $key being created or updated.",2);
				continue;
				}

			if ($this->objtype!="session" AND ($key=="siteid" OR ($key=="folderid" AND $this->objtype!="folder") OR $key=="sitememberid")) {
				berror("updateObject() continuing on siteid, folderid, or sitememberid (objtype!=session). ($key)",2);
				continue;
				}

			if ( $this->{$key}=="" ) {
				berror("updateObject() continuing for this->$key='' (".$this->{$key}.") and postvars[$key] is '' also. (<b>".$postvars[$key]."</b>)",2);
				continue;
				}
			else {
				berror("updateObject() setting newobject[$key]=".$this->{$key}.".",2);
				$newobject[$key]= $this->{$key};
				$queryvalues= $queryvalues.", $key= '$newobject[$key]'";
				}
			}		

		// process the update query
		$query= "UPDATE $this->objtype SET $queryvalues WHERE id='$this->id' ";
		$result= mysql_query($query);
		if (!$result) {
			berror("updateObject(): update query failed to get a result. $query; (".mysql_error().")",1);
			}
		else $success=1;

		// unencode object
		$this->mySQLToBerylium();
                
                // get the date and time in case we need it.
                $this->updated= date("Y-m-d H:i:s");

		berror("updateObject(): [bold:update successful]. $query;",1);
		return $success;
		}

	function selectObject($query) {
		// selectObject runs a query on the database and returns an $objectarray of results
		//    as well as loading the first object into $this
		berror("selectObject() called by ".get_class($this)." id#$this->id ($this->name) with query=$query;",1);
		$success=0;

		// connect to database
		$session= $GLOBALS[session];
		if (!$session->dbconnection) $GLOBALS[session]->dbconnection= dbconnect();
		

		// get the result set
		$result= mysql_query($query);
		if (!$result) {
			berror("selectObject(): query failed to get a result: (".mysql_error().")",1);
			return 0;
			}

		// take first result and load it into $this object
		$objectarray[0]= mysql_fetch_array($result);
		if (is_array($objectarray[0])) {
			foreach($objectarray[0] as $key=>$value) {
				if (is_int($key)) continue;
				$this->{$key}= $value;
				}
			$success= 1;
			$n=1;

			while ($objectarray[$n]= mysql_fetch_array($result)) {
				$n= $n+1;
				$success= $n;
				}
			}

		else {
			berror("selectObject(): Empty result found at $result. ",1);
			$success= 0;
			}
		
		$this->listsize= $success;
		return $objectarray;
		}

	function getMySQLColumns($objtype="BeryliumObject") {
		// gets the mySQL column_definition from a BeryliumObject of type $objtype for makeTable()
		//   allows objects to define their own SQL tables
		berror("getMySQLColumns() called by ".get_class($this)." id#$this->id ($this->name).",1);

		//create a temporary object
		$command= "\$temp= new $objtype;";
		berror("getMySQLColumns(): Command=$command",2);
		eval("$command");

		//unpack the ^ delimited columns, then use that array to build the column_definition 
		$columns= bunpack($temp->columns);
		while (list ($key, $val)= each($columns)) {
			$column_definition= $column_definition.", $key $val";
			}
		$column_definition= substr($column_definition,2);

		// recurse back through parent classes
		if ($parentclass= get_parent_class($temp)) {
			$prepend= $temp->getMySQLColumns($parentclass);
			if ($prepend!="" AND $column_definition!=" ") $column_definition= $prepend.", ".$column_definition;
			elseif ($prepend!="") $column_definition= $prepend;
			}

		// return the column definition
		berror("getMySQLColumns(): Returning column definition: $column_definition",2);
		return $column_definition;
		}

	function makeTable() {
		// initializes a table to hold this type of object
		berror("makeTable() called by ".get_class($this)." id#$this->id ($this->name).",1);

		if (!$this->classname= get_class($this)) {
			berror("makeTable(): can't figure out classname of this '$this->objtype' object. Sad face.",0);
			}

		// connect to database
		$session= $GLOBALS[session];
		if (!$session->dbconnection) $GLOBALS[session]->dbconnection= dbconnect();

		// use getMySQLColumns() to build the Create Table query		
		$query="CREATE TABLE ".$this->objtype." (".$this->getMySQLColumns($this->classname).") ";
		$result= mysql_query($query);

		if (!$result) {
			berror("makeTable(): Error on Create Table query= '$query'; (".mysql_error().")",1);
			$success=0;
			}
		else $success=1;

		return $success;
		}

	function checkDuplicate($field="name") {
		// checks the database for a similar object with the same name (or some other field)
		berror("checkDuplicate() called on '$field' by $this->objtype id#$this->id ($this->name) in site=$this->siteid and folder=$this-folderid.",1);

		// sites must have unique names, period
		if ($this->objtype=="site" and $field=="name") $nonfolder="";

		// folders, must be named uniquely sitewide
		if ($this->objtype=="folder" AND $field=="name") $nonfolder= "AND siteid='$this->siteid' ";

		// otherwise name must be unique to containing folder only
		else $nonfolder= "AND folderid='$this->folderid' ";

		// build query -- it's okay if $this->id=="" because then it won't be equal to any other ids, will it?
		$query= "SELECT id,name,status FROM $this->objtype WHERE id!='$this->id' AND $field='".$this->{$field}."' $nonfolder ";
		$result= mysql_query($query);

		berror("checkDuplicate() is checking for duplicate $this->objtype using query=$query",1);

		if ($duplicate= @mysql_fetch_array($result)) {
			berror("checkDuplicate(): A $this->objtype with the same $field already exists here. It is $this->objtype id#$duplicate[id] with status=$duplicate[status]. Please use the back button to change the name of the object you are trying to create or edit.",1);
			$found=$duplicate;
			}
		else $found=0;

		return $found;
		}


// encode/decode, translation, etc.
	function beryliumToHtml() {
		// processes text fields for html display with bml parsed -- not reversible!
		// theory: removes all html tags, curly brackets "{}" and doublequotes from user input, converts newlines to <br>, and parses bml
		// warning: only works with text in ISO-Latin-1 charset!!!
		berror("beryliumToHtml() called by ".get_class($this)." id#$this->id ($this->name).",1);

		$objectproperties= get_object_vars($this);
		foreach ($objectproperties as $key=>$value) {
			if (!is_string($value)) continue;
			$value= preg_replace("/[{}]/","|",$value);
			$this->{$key}= bpostbml(bparsebml(nl2br(htmlentities($value))));
			}

		return "text/html";
		}

	function beryliumToForm() {
		// processes text fields for html display -- reversible
		// theory: removes all html tags and doublequotes from user input (" becomes &quot;)
		// warning: only works with text in ISO-Latin-1 charset!!!	
		berror("beryliumToForm() called by ".get_class($this)." id#$this->id ($this->name).",1);

		$objectproperties= get_object_vars($this);
		foreach ($objectproperties as $key=>$value) {
			if (!is_string($value)) continue;
			$this->{$key}= htmlentities($value);
			}

		return "text/html";
		}

	function formToBerylium() {
		// reverts html-encoded text back to regular chars (&quot; becomes ")
		// warning: only works with text in ISO-Latin-1 charset!!!	
		berror("formToBerylium() called by ".get_class($this)." id#$this->id ($this->name).",1);

		// reverse the htmlentities translation table...
		$trans = get_html_translation_table(HTMLENTITIES);
		$trans = array_flip ($trans);

		$objectproperties= get_object_vars($this);
		foreach ($objectproperties as $key=>$value) {
			if (!is_string($value)) continue;
			$this->{$key}= strtr($value, $trans);
			}

		return "text/plain";
		}

	function beryliumToMySQL() {
		// processes all fields for insertion into database -- reversible
		// theory: escapes quotes and doublequotes in user input so they don't break query strings (" becomes \")
		berror("beryliumToMySQL() called by ".get_class($this)." id#$this->id ($this->name) (path=$this->path).",1);

		$objectproperties= get_object_vars($this);
		foreach ($objectproperties as $key=>$value) {
			if (!is_string($value)) continue;
			$this->{$key}= addslashes($value);
			}

		return "text/php";
		}

	function mySQLToBerylium() {
		// removes quoting
		// theory: unescapes quotes and doublequotes in user input (\" becomes ")
		berror("mySQLToBerylium() called by ".get_class($this)." id#$this->id ($this->name).",1);

		$objectproperties= get_object_vars($this);
		foreach ($objectproperties as $key=>$value) {
			if (!is_string($value)) continue;
			$this->{$key}= stripslashes($value);
			}

		return "text/plain";
		}

	function timeFormat($format='D M j, Y, \a\t H:i:s') {
		// converts $this->created and $this->modified to a UNIX timestamp, then into $format
		berror("timeFormat() called with '$format' by ".get_class($this)." id#$this->id ($this->name).",1);

		$temp= $this->created;
		$created= mysql_timestamp($this->created);
		$this->created= date($format, $created);
		berror("timeFormat(): $temp was translated as $this->created.",2);

		$updated= mysql_timestamp($this->updated);
		$this->updated= date($format, $updated);

		return 1;
		}

	function parseCBML($field){
		// finds and translates commands written in curly-bracket markup language {cbml} within contexts
		// see cbml-engine.php
		// note that field is $this->context->field, not $this->field -- why? not quite sure yet.
		berror("[color:#ff6600;parseCBML() called with '$field' by $this->objtype id#$this->id ($this->name).]",1);

		$site= $GLOBALS[site];
		$folder= $GLOBALS[folder];
		$session= $GLOBALS[session];
		$containers= $GLOBALS[containers];
                $object= $GLOBALS[object];
                

		$code= $this->context->{$field};
		berror("[color:#ff6600;parseCBML() code=$code.]",2);
		berror("parseCBML(): policy report: this->publicwhere= $session->publicwhere.",1);

		// assume markup will be found
		$found=1;
		$end=0;
	
		//begin parsing loop
		while ($found) {
			// sanity time
			if ($end>=strlen($code)) {
				berror("parseCBML(end ($end) is greater than the string length of code, exiting loop.)",1);
				$found=0;
				continue;
				}
	
			// find command start
			$start= strpos($code, "{");
			if ($start===false) {
				berror("parseCBML(no more { found.)",1);
				$found=0;
				continue;
				}
			
			// look for at least one right bracket, otherwise this is madness!
			$endmost= strrpos($code, "}");
			if ($endmost===false) {
				berror("parseCBML(no endmost }, this is madness.)",1);
				$found=0;
				continue;
				}
	
			// check for command
			$commandend= strpos($code, ":", $start);
			if ($commandend===false) {
				$end= $start+1;
				continue;
				}
	
			// if the alleged "command" ends after the last right bracket, it isn't one.
			if ($commandend>$endmost) {
				berror("parseCBML(commandend ($commandend) is further out than the endmost } at $endmost, exiting.)",1);
				$found=0;
				continue;
				}
	
			// find a likely endpoint for this tag
			$anend= strpos($code, "}", $start);
			berror("parseCBML(Found a likely endpoint } at $anend)",1);
	
			// is there lunacy here? something like "[bold=boldme] blah blah [ital:italic]"???
			if ($commandend>$anend) {
				berror("parseCBML(lunacy here, commandend ($commandend) is greater than anend ($anend), trying another loop.)",1);
				$end= $start+1;
				continue;
				}
	
	
			// nested markup is not allowed in cbml!
			/*
			$next= strpos($code, "{", $start+1);
			berror("parseCBML(Next { at $next)",1);
	
			if ($next<$anend AND !($next===false)) {
				$nestlen= $endmost-$start;
				$nest= substr($code, $start+1, $nestlen);
				$code= substr($code, 0, $start+1).bparsebml($nest).substr($code, $endmost+1);
				}
			// nested markup was not allowed in cbml!
			*/
	
	
			// okay, now we can find the one living and true end
			$end= strpos($code, "}", $start);
			if ($end===false) {
				$found=0;
				continue;
				}
			berror("parseCBML(true end found at $end.)",1);

	
			// check for command again
			$commandend= strpos($code, ":", $start);
			if ($commandend===false) continue;
			berror("parseCBML(commandend found at $commandend.)",1);

	
			// and again, if the : is outside the end bracket, forget it.
			if ($commandend>$end) {
				berror("parseCBML(commandend greater than end, forget it.)",1);
				continue;
				}
			
			// get command
			$commandlen= $commandend-$start-1;
			$command= substr($code, $start+1, $commandlen);
			if ($command=="") continue;
	
			// define tag
			$taglen= $end-$commandend-1;
			$tag= substr($code, $commandend+1, $taglen);

		//parse the tag for variables.
		$commandv= "\$tag= \"$tag\";";
		//$this->applyPolicy();
		berror("parseCBML() parsing $tag for variables: $commandv. ($this->imageid)",1);
		eval($commandv);
	
			// with $command and $tag in hand, run the bmlengine
			include("$GLOBALS[beryliumroot]/code/cbml-engine.php");
	
			// make the substitution
			$code= substr($code, 0, $start).$tag.substr($code, $end+1);
			berror("parseCBML(substitution made, continuing.)",1);
			
			} //end while
	
		$this->context->{$field}= $code;
		}
		


// requestable methods (view, save, list, etc.)

	function view() {
		// get object from the database, get context, render -- works for edit, too!
		// requires an object with objtype and name, and a folder with id from $containers[0] and $session->request->(method,format)
		berror("view() called by '$this->objtype' object id#$this->id ($this->name).",1);
		
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$folder= $GLOBALS[folder];
		$request= $GLOBALS[session]->request;

		// get object context
		$this->getContext($request->method, $request->format);

		// impose a folderid requirement if this isn't a site or folder...
		if ($this->objtype!="folder" AND $this->objtype!="site") $notafolder= "AND obj.folderid='$folder->id' ";

		// apply policy
		if ($this->objtype!="policy") $query= "SELECT * FROM $this->objtype AS obj WHERE obj.name='$this->name' AND obj.siteid='$site->id' $notafolder $session->statuswhere $session->publicwhere $session->orderby $session->limit";
		else $query= "SELECT * FROM $this->objtype AS obj WHERE obj.name='$this->name' AND obj.siteid='$site->id' $notafolder $session->statuswhere $session->orderby $session->limit";
		if ($this->objtype=="context") {
			$this->parseName();
			$this->selectContext();
			}
		else $this->selectObject($query);
		if ($this->listsize) {
			berror("view() found '$this->objtype' object id#$this->id ($this->name), now rendering.",1);
			$this->preProcess();
			$this->css($GLOBALS[session]);
			if ($this->objtype!="site" AND $this->objtype!="folder") $this->header($GLOBALS[session]);
			if ($this->showNull) $this->nullObject($GLOBALS[session]);
			else $this->template($GLOBALS[session]);
			if ($this->objtype!="site" AND $this->objtype!="folder") $this->footer($GLOBALS[session]);
			$this->postProcess();
			}
		elseif ($this->name=="0") {
			$query= "SELECT * FROM $this->objtype AS obj WHERE 1 $notafolder $session->whereposted $session->wherepublic $session->orderby $session->limit";
			$this->selectObject($query);
			if ($this->listsize) {
				$output= blist($query,"list");
				boutput($output, $GLOBALS[session]);
				}
			else {
				berror("view() didn't find any (using id=0) '$this->objtype' object id#$this->id, null object.",1);
				$this->preProcess();
				$this->css($GLOBALS[session]);
				if ($this->objtype!="site" AND $this->objtype!="folder") $this->header($GLOBALS[session]);
				$this->nullObject($GLOBALS[session]);
				if ($this->objtype!="site" AND $this->objtype!="folder") $this->footer($GLOBALS[session]);
				$this->postProcess();
				}

			}
		elseif (is_numeric($this->name) && $this->name > 0) {
			$this->id= $this->name;
			$query= "SELECT * FROM $this->objtype AS obj WHERE obj.id='$this->id' $notafolder $session->whereposted $session->wherepublic $session->orderby $session->limit";
			$this->selectObject($query);
			if ($this->listsize) {
				berror("view() found (using id) '$this->objtype' object id#$this->id ($this->name), now rendering.",1);
				$this->preProcess();
				$this->css($GLOBALS[session]);
				if ($this->objtype!="site" AND $this->objtype!="folder") $this->header($GLOBALS[session]);
				if ($this->showNull) $this->nullObject($GLOBALS[session]);
				else $this->template($GLOBALS[session]);
				if ($this->objtype!="site" AND $this->objtype!="folder") $this->footer($GLOBALS[session]);
				$this->postProcess();
				}
			else {
				berror("view() didn't find (using id) '$this->objtype' object id#$this->id, null object.",1);
				$this->preProcess();
				$this->css($GLOBALS[session]);
				if ($this->objtype!="site" AND $this->objtype!="folder") $this->header($GLOBALS[session]);
				$this->nullObject($GLOBALS[session]);
				if ($this->objtype!="site" AND $this->objtype!="folder") $this->footer($GLOBALS[session]);
				$this->postProcess();
				}
			}
		else {
			berror("view() didn't find '$this->objtype' object id#$this->id ($this->name), null object.",1);
			$this->preProcess();
			$this->css($GLOBALS[session]);
			if ($this->objtype!="site" AND $this->objtype!="folder") $this->header($GLOBALS[session]);
			$this->nullObject($GLOBALS[session]);
			if ($this->objtype!="site" AND $this->objtype!="folder") $this->footer($GLOBALS[session]);
			$this->postProcess();
			}
		
		return 1;
		}

	function save() {
		// build object from HTTP_POST_VARS, get context, process, save, postprocess
		// requires $this->objtype, POST_VARS with optional id, and $session->request->(method,format)
		berror("save() called by '$this->objtype' object id#$this->id ($this->name) (path=$this->path).",1);
		
		$posto= $GLOBALS[HTTP_POST_VARS];
		$request= $GLOBALS[session]->request;

		if ($this->objtype!="folder") {
			// load POST data into object (folders are special)
			foreach ($posto as $key=>$value) {
				$this->{$key}= $value;
				}
			}

		// has the "Create New" button been pressed? 
		if ($this->Submit2=="create new") {
			$this->id= "";
			}

		// get context for pre- and post-processing and role control
		$this->getContext($request->method, $request->format);

		$this->preProcess();
		if ($this->objtype!="site" AND $this->objtype!="folder") $this->header($GLOBALS[session]);
		$this->template($GLOBALS[session]);
		if ($this->objtype!="site" AND $this->objtype!="folder") $this->footer($GLOBALS[session]);
		if ($this->id!="") $this->updateObject();
		else $this->insertObject();
		$this->postProcess();
		
		return 1;
		}


// object-fu: makeAlias, copy, move

	function move($newfolder, $newparent="") {
		berror("move(): called by $this->objtype $this->name (#$this->id), moving into $folder->name.",1);
		
		// must be called with at least object $newfolder
		if (!is_object($newfolder) || $newfolder->id=="") {
			berror("move(): called with missing or unloaded newfolder object.",1);
			return 0;
			}
		if (is_object($newparent) && $newparent->id=="") {
			$withparent= 1;
			berror("move(): called with newparent ($newparent->objtype $newparent->name) but it has no id.",1);
			return 0;
			}
			
		// change folderid (and parentid and parentobjtype)
		$this->folderid= $newfolder->id;
		if ($withparent) {
			$this->parentid= $newparent->id;
			$this->parentobjtype= $newparent->objtype;
			}
			
		// move any associated files
		if ($this->objtype=="folder") {
			// folders are special and ugly-- they need all of their contents updated as well...
			berror("move(): called on a folder object. Sorry, we're not equipped to handle that yet.",0);
			return 0;
			}
			
		// if image, update path and uri
		if ($this->objtype=="image") {
			if (!file_exists($this->path)) {
				berror("move(): not moving any files: $this->path does not exist.",1);
				}
			else {
				$dir= dirname($newfolder->name);
				$base= basename($this->path);
				$dotpos= strrpos($base, ".");
				if ($dotpos) {
					$ext= substr($base,$dotpos);
					}
				$renamepath= "$dir/$newname$ext";
				berror("move(): set to move image file $this->path to $renamepath.",1);
				if (rename($this->path, $renamepath)) {
					berror("move(): file move successful.",1);
					}
				else {
					berror("move(): unable to rename image file: from $this->path to $renamepath. Not updated.",0);
					return 0;
					}
				}
			
			// get new path and uri information
			if ($renamepath) {
				$this->path= $renamepath;
				$this->uri= $folder->name."/image-$newname$ext";
				}
			else {
				$base= basename($temp->path);
				$dotpos= strrpos($base, ".");
				if ($dotpos) {
					$ext= substr($base,$dotpos);
					}
				$this->path= dirname($temp->path)."/$newname$ext";
				$this->uri= dirname($temp->uri)."/image-$newname$ext";
				}
			berror("Image->save(): rename finishing. New path is ($this->path), new uri is ($this->uri).",1);
			}
			
		return 1;
		}


// context-related methods

	function getContext($method="", $format="") {
		// getContext() looks back through the containers array for just the right context to use: the first one it finds.
		// If context is not found in any of these places, the object cannot be rendered.
		berror("[color:#0033CC;getContext() called by $this->objtype id $this->id ($this->name).] (local method='$method', local format='$format')",1);
		
		// use the following to turn db lookup of contexts on(1)/off(0)
		$dbcontexts=0;

		$success= 0;
		$session= $GLOBALS[session];
		$sitemember= $GLOBALS[sitemember];
		$containers= $GLOBALS[containers];
		$folder= $GLOBALS[folder];
		$context= new Context;

		if ($method=="") $method= $session->request->method;
		if ($format=="") $format= $session->request->format;

		// turn down the errorLevel for all these queries
		$olderror= $session->errorLevel;
		//$GLOBALS[session]->setErrorLevel(0);
		berror("<blockquote>",1);

		if ($dbcontexts) {
			// class-specific, role-specific, up through folders
			if ($sitemember->role!="") {
				foreach ($containers as $key=>$value) {
					if ($success) continue;
					$folderid= $value->id;
					$query= "SELECT * FROM context WHERE method='$method' AND format='$format' AND role='$sitemember->role' AND classname='$this->objtype' AND folderid='$folderid' AND status='posted' ";
					$context->selectObject($query);
					if ($context->id!="") {
						$success=1;
						}
					}	
				}
	
			// class-specific, role-specific (editor becomes member), up through folders
			if (!$success AND $sitemember->role=="editor") {
				foreach ($containers as $key=>$value) {
					if ($success) continue;
					$folderid= $value->id;
					$query= "SELECT * FROM context WHERE method='$method' AND format='$format' AND role='member' AND classname='$this->objtype' AND folderid='$folderid' AND status='posted' ";
					$context->selectObject($query);
					if ($context->id!="") {
						$success=1;
						}
					}	
				}
	
			// class-specific, role-anonymous, up through folders
			if (!$success) {
				foreach ($containers as $key=>$value) {
					if ($success) continue;
					$folderid= $value->id;
					$query= "SELECT * FROM context WHERE method='$method' AND format='$format' AND role='anonymous' AND classname='$this->objtype' AND folderid='$folderid' AND status='posted' ";
					$context->selectObject($query);
					if ($context->id!="") {
						$success=1;
						}
					}
				}
	
			// class-generic, role-specific, up through folders
			if (!$success) {
				if ($sitemember->role!="") {
					foreach ($containers as $key=>$value) {
						if ($success) continue;
						$folderid= $value->id;
						$query= "SELECT * FROM context WHERE method='$method' AND format='$format' AND role='$sitemember->role' AND classname='generic' AND folderid='$folderid' AND status='posted' ";
						$context->selectObject($query);
						if ($context->id!="") {
							$success=1;
							}
						}	
					}
				}
	
			// class-generic, role-generic, up through folders
			if (!$success) {
				foreach ($containers as $key=>$value) {
					if ($success) continue;
					$folderid= $value->id;
					$query= "SELECT * FROM context WHERE method='$method' AND format='$format' AND role='anonymous' AND classname='generic' AND folderid='$folderid' AND status='posted' ";
					$context->selectObject($query);
					if ($context->id!="") {
						$success=1;
						}
					}
				}
			} // end if ($dbcontexts)
		else {
			// look for contexts in filesystem...
			$success= 0;
			if ($folder->flavor!="folder") $context->flavorpath= "$folder->flavor/";
			
			//first pass with folder->flavor...
			$contextpath= $GLOBALS['beryliumroot']."/code/contexts/$context->flavorpath";
			
				// class-specific, role specific
				$context->name= "$this->objtype-$method-$sitemember->role-$format";
				berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
				$path= $contextpath.$context->name;
				if (file_exists($path)) {
					$context->uploadContext($path);
					$success=1;
					}
					
				// class-specific, role editor
				if (!$success && $sitemember->role=="admin") {
					$context->name= "$this->objtype-$method-editor-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
				
				// class-specific, role member
				if (!$success && ($sitemember->role=="editor" || $sitemember->role=="admin")) {
					$context->name= "$this->objtype-$method-member-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
								
				// class-specific, role anonymous
				if (!$success && $sitemember->role!="anonymous") {
					$context->name= "$this->objtype-$method-anonymous-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
					
				// class-generic, role anonymous
				if (!$success) {
					// try generic objtype...
					$context->name= "generic-$method-anonymous-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
					
			// second pass without folder->flavor
			if (!$success && $context->flavorpath) {
				$contextpath= $GLOBALS['beryliumroot']."/code/contexts/";
			
				// class-specific, role specific
				$context->name= "$this->objtype-$method-$sitemember->role-$format";
				berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
				$path= $contextpath.$context->name;
				if (file_exists($path)) {
					$context->uploadContext($path);
					$success=1;
					}
					
				// class-specific, role editor
				if (!$success && $sitemember->role=="admin") {
					$context->name= "$this->objtype-$method-editor-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
				
				// class-specific, role member
				if (!$success && ($sitemember->role=="editor" || $sitemember->role=="admin")) {
					$context->name= "$this->objtype-$method-member-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
								
				// class-specific, role anonymous
				if (!$success && $sitemember->role!="anonymous") {
					$context->name= "$this->objtype-$method-anonymous-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
					
				// class-generic, role anonymous
				if (!$success) {
					// try generic objtype...
					$context->name= "generic-$method-anonymous-$format";
					berror("getContext(): Going to look for root context file at $contextpath$context->name.",1);
					$path= $contextpath.$context->name;
					if (file_exists($path)) {
						$context->uploadContext($path);
						$success=1;
						}
					}
				}
			}
			
		// turn errorLevel back up
		berror("</blockquote>",1);
		//$GLOBALS[session]->setErrorLevel($olderror);

		// if found, return the context object
		if ($success) {
			berror("getContext(): [bold: Found '".$context->classname."' context] (#$context->id) in folder #".$context->folderid." (with role='".$context->role."' and method='".$context->method."').",1);
			$this->context= $context;
			return 1;
			}
		else	{
			// nothing found, exit unsuccessfully
			berror("getContext(): [bold:No valid context] was found for the $this->objtype you are trying to access.",1);
			$this->context->nullobject= "<table width='50%' cellpadding=8 align='center'><tr><td>Sorry, you do not have permission to access this page.<br><br>(You may need to log in...)</td></tr></table>";
			return 0;
			}
		}


	function preprocess() {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];
		$object= $GLOBALS[object];
		$containers= $GLOBALS[containers];
		berror("preprocess() called by $this->objtype #$this->id ($this->name) (for $object->objtype page ($object->id))",1);

		if ($this->context->preprocess!="") {
			if ($session->errorLevel==2) {
				$this->context->preprocesshtml= htmlentities($this->context->preprocess);
				berror("preprocess() processing <blockquote><tt>".$this->context->preprocesshtml."</tt></blockquote>",2);
				}
			eval($this->context->preprocess);
			}

		return 1;
		}

	function css(&$outobject) {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];

		berror("css() called by $this->objtype #$this->id ($this->name) to be sent to $outobject->objtype id#$outobject->id ($outobject->name).",1);
		if ($this->context->css!="") {
			$css= $this->context->css;
			berror("css() processing this: ".htmlentities($css),2);
			$command= "\$output= \"$css\";";
			eval($command);
			}

		$GLOBALS[session]->css= $session->css.$output;
		return 1;
		}

	function header(&$outobject) {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];
		$object= $GLOBALS[object];

		berror("header() called by $this->objtype #$this->id ($this->name) to be sent to $outobject->objtype id#$outobject->id ($outobject->name).",1);
		if ($this->context->header!="") {
			$this->parseCBML("header");
			$header= str_replace('"','\"',$this->context->header);
			berror("header() processing this: ".htmlentities($header),2);
			$command= "\$output= \"$header\";";
			eval($command);
			}

		if ($this->objtype=="site" && $outobject->objtype=="session") {
			$GLOBALS[session]->output[0]= $output;
			berror("[color:#66FF00;header() inserted chunk 0 (site header) into session id#$session->id]",1);
			}
		elseif ($this->objtype=="folder" && $outobject->objtype=="session") {
			$GLOBALS[session]->output[1]= $output;
			berror("[color:#66FF00;header() inserted chunk 1 (folder header) into session id#$session->id]",1);
			}
		else {
			boutput($output,$outobject);
			}

		return 1;
		}

	function template(&$outobject) {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];
		$object= $GLOBALS[object];

		berror("template() called by $this->objtype #$this->id ($this->name) to be sent to $outobject->objtype id#$outobject->id ($outobject->name).",1);
		if ($this->context->template!="") {
			$this->parseCBML("template");
			$template= str_replace('"','\"',$this->context->template);
			berror("template() processing this: ".htmlentities($template),2);
			$command= "\$output= \"$template\";";
			eval($command);
			}

		boutput($output,$outobject);
		return 1;
		}

	function nullObject(&$outobject) {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];
		$object= $GLOBALS[object];

		berror("nullObject() called by $this->objtype #$this->id ($this->name) to be sent to $outobject->objtype id#$outobject->id ($outobject->name).",1);
		if ($this->context->nullobject!="") {
			$this->parseCBML("nullobject");
			$nullobject= str_replace('"','\"',$this->context->nullobject);
			berror("nullObject() processing this: ".htmlentities($nullobject),2);
			$command= "\$output= \"$nullobject\";";
			eval($command);
			}

		boutput($output,$outobject);
		return 1;
		}

	function listRow(&$outobject) {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];
		$object= $GLOBALS[object];

		berror("listrow() called by $this->objtype #$this->id ($this->name) to be sent to $outobject->objtype id#$outobject->id ($outobject->name).",1);
		if ($this->context->listrow!="") {
			$this->parseCBML("listrow");
			$listrow= str_replace('"','\"',$this->context->listrow);
			berror("listrow() processing this: ".htmlentities($listrow),2);
			$command= "\$output= \"$listrow\";";
			eval($command);
			}

		boutput($output,$outobject);
		return 1;
		}

	function footer(&$outobject) {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];
                $object= $GLOBALS[object];
		$context= $this->context;

		berror("footer() called by $this->objtype #$this->id ($this->name) to be sent to $outobject->objtype id#$outobject->id ($outobject->name).",1);
		if ($this->context->footer!="") {
			$this->parseCBML("footer");
			$footer= str_replace('"','\"',$this->context->footer);
			berror("footer() processing this: ".htmlentities($footer),2);
			$command= "\$output= \"$footer\";";
			eval($command);
			}

		boutput($output,$outobject);
		return 1;
		}

	function postprocess() {
		$session= $GLOBALS[session];
		$site= $GLOBALS[site];
		$member= $GLOBALS[member];
		$folder= $GLOBALS[folder];
		$sitemember= $GLOBALS[sitemember];
		$containers= $GLOBALS[containers];
		$object= $GLOBALS[object];
		berror("postprocess() called by $this->objtype #$this->id ($this->name)",1);

		if ($this->context->postprocess!="") {
			berror("postprocess() processing ".$this->context->postprocess,2);
			eval($this->context->postprocess);
			}

		return 1;
		}


// utilities methods

	function getPropertiesList() {
		// generates an html properties list for the object
		berror("getPropertiesList() called by ".get_class($this)." id $this->id ($this->name).",1);

		$array= get_object_vars($this);
		$list= "<hr><b>".get_class($this)." values:</b><blockquote>";
		while (list($key, $val)= each($array)) {
			if (is_int($key)) continue;
			$list= $list."$key => $val<br>";
			if (is_object($val) and $key!="context") $list= $list.$val->getPropertiesList();
			}
		$list= $list."---End of ".get_class($this)." #$this->id ---</blockquote>";

		return $list;
		}

	function getBaseURL() {
		// returns the urls, uri, and filesystem path that reference this object
		$session= $GLOBALS[session];

		if ($session->request->connection=="https") $scriptname= "/berylium";
		else $scriptname= "/berylium";

		$urlbase= $session->request->connection."://".$GLOBALS[site]->name.$scriptname;
		
		// get parent folder of this object
		if ($this->objtype=="site") {
			$url= $urlbase."/";
			$idurl= $urlbase."/site-$this->id.".$session->request->format;
			$path= $GLOBALS[beryliumroot]."/files/$this->name";
			$uri= $scriptname;
			}
		else {
			$query= "SELECT name FROM folder WHERE id='$this->folderid' ";
			$this->parentfolder= blookup($query);
			if ($this->parentfolder==0) {
				berror("getBaseURL couldn't find parent folder of $this->objtype #$this->id ($this->name): folder id$this->folderid.",1);
				return 0;
				}

			// kludge to allow insertion of ending slash in foldername...
			if ($this->parentfolder[name]=="/") $this->parentfolder[name]="";
			
			// urlencode folder and object names
			//$this->parentfolder[name]= urlencode($this->parentfolder[name]);
			$this->urlname= urlencode($this->name);
	
			// baseUrl
			// named url
			// current connection protocol and request format are used
			if ($this->objtype=="folder") {
				// like: http://sitename/berylium/folder/name/
				if ($this->name=="/") $url= $urlbase.$this->name;
				else $url= $urlbase.$this->name."/";
				}
			else {
				// like: http://sitename/berylium/folder/name/document-docname.xml
				$url= $urlbase.$this->parentfolder[name]."/".$this->objtype."-".$this->urlname.".".$session->request->format;
				}

			// idUrl
			// idurl -- good for unnamed things or things that might not have url-friendly names
			// like: http://sitename/berylium/folder/name/document-18.html
			$idurl= $urlbase.$this->parentfolder[name]."/".$this->objtype."-".$this->id.".".$session->request->format;
			
			// path
			// path in filesystem
			if ($this->objtype=="folder") {
				// like: /home/berylium/files/sitename/folder/name
				$path= $GLOBALS[beryliumroot]."/files/".$GLOBALS[site]->name.$this->name;
				}
			else {
				// like: /home/berylium/files/sitename/folder/name/imagename.jpg
				// or: // like: /home/berylium/file/folder/name/docname.html
				if ($this->format=="") $this->format= ".html";
				$path= $GLOBALS[beryliumroot]."/files/".$GLOBALS[site]->name.$this->parentfolder[name]."/".$this->urlname.$this->format;
				}
			
			// uri
			// uri is a reference to the object in its native format (html unless folder or otherwise indicated)
			// no connection protocol is indicated
			if ($this->objtype=="folder") {
				if ($this->name=="/") $uri= $scriptname."/index.html";
				else $uri= $scriptname.$this->name."/index.html";
				}
			else {
				if ($this->format=="") $this->format= ".html";
				$uri= $scriptname.$this->parentfolder[name]."/".$this->objtype."-".$this->urlname.$this->format;
				}
			}
		
		berror("getBaseURL() for $this->objtype #$this->id ($this->name):<blockquote>",1);
		berror("getBaseURL() named url=".$url,1);
		berror("getBaseURL() and id url=".$idurl,1);
		berror("getBaseURL() and path=".$path,1);
		berror("getBaseURL() and uri=".$uri,1);
		berror("</blockquote>",1);

		$this->baseURL= $url;
		$this->baseUrl= $url;
		$this->idURL= $idurl;
		$this->idUrl= $idurl;
		$this->path= $path;
		$this->uri= $uri;
		return 1;
		}



	function oldloadObject() {	// DEPRECATED!! ****************************************************
		// loadObject refreshed $this with data from the database. Requires $this->objtype and either $this->id or ($this->name and $this->folderid)
		berror("<b>DEPRECATED!</b> loadObject() called by ".get_class($this)." id#$this->id ($this->name, in folder #$this->folderid).",1);

		// connect to database if not already
		$session= $GLOBALS[session];
		if (!$session->dbconnection) $GLOBALS[session]->dbconnection= dbconnect();

		// pick a query scheme
		if ($this->id!="" AND $this->objtype!="") {
			$query= "SELECT * FROM $this->objtype WHERE id=$this->id $this->queryExtras";
			}
		elseif ($this->name!="" AND $this->folderid!="" AND $this->objtype!="") {
			$query= "SELECT * FROM $this->objtype WHERE name='$this->name' AND folderid='$this->folderid' $this->queryExtras";
			}
		elseif ($this->name!="" AND $this->objtype=="folder" OR $this->objtype=="site") {
			$query= "SELECT * FROM $this->objtype WHERE name='$this->name' $this->queryExtras";
			}
		else {
			berror("<font color='#FF0000'>loadObject(): called by ".get_class($this)." without enough details.</font>",1);
			}

		$this->selectObject($query);

		return 1;
		}



	function viewObject() {	// DEPRECATED!! ****************************************************
		// default view context; to do: render to session->output...
		berror("<b>DEPRECATED!</b> viewObject() called by ".get_class($this)." id $this->id ($this->name).",1);

		$array= get_object_vars($this);
		print "<br><b>".get_class($this)." values:</b><blockquote>";
		while (list($key, $val)= each($array)) {
			if (is_int($key)) continue;
			print "$key => $val<br>";
			if (is_object($val)) $val->viewObject();
			}
		print "---End---</blockquote>";
		}



	function createObject() {	// DEPRECATED!! ****************************************************
		berror("<b>DEPRECATED!</b> createObject() called by ".get_class($this)." id $this->id ($this->name), to create a new $this->objtype.",1);

		$this->id="";
		$this->editObject();
		}

	function editObject() {	// DEPRECATED!! ****************************************************
		// generates an HTML form for the raw editing of an object
		berror("<B>DEPRECATED</B> editObject() called by ".get_class($this)." id $this->id ($this->name).",1);

		// connect to database
		$session= $GLOBALS[session];
		if (!$session->dbconnection) $GLOBALS[session]->dbconnection= dbconnect();

		// get table description so we know what bits of this object we can save
		$query= "DESCRIBE $this->objtype ";
		$result= mysql_query($query);
		$a=1;
		while ($columnarray= mysql_fetch_array($result)) {

			// what are some things that can't change? 
			$continuelist= array("id", "created", "updated", "sessionid");
			foreach ($continuelist as $k=>$v) {
				if ($v==$columnarray[Field]) {
					$ignorefield=1;
					berror("editObject(): ignoring db field $columnarray[Field].",1);
					}
				}

			$thisfield= $columnarray[Field];
			if ($ignorefield==0) $tablecolumn[$thisfield]= $a;
			$ignorefield=0;
			$a++;
			}

		// hidden form variables (objtype and id if available)
		$hiddenvars= "<input type='hidden' name='objtype' value='$this->objtype'>";
		if ($this->id) $hiddenvars= $hiddenvars."<input type='hidden' name='id' value='$this->id'>";

		// print the form
		print "<form method='post' action='?method=update'> $hiddenvars
			  <table width='400' border='0' cellspacing='0' cellpadding='5'>
				<tr><td colspan='2'><b>Edit $this->objtype id#$this->id</b></td></tr>";

		foreach ($tablecolumn as $key=>$val) {
			print "
			   <tr valign='top'> 
			      <td align='right'>$key:</td>
			      <td> 
			        <input type='text' name='$key' size='24' maxlength='255' value='".$this->{$key}."'>
			      </td>
			    </tr>
				";
			}

		print "
			    <tr> 
			      <td width='20%'>&nbsp;</td>
			      <td> 
			        <input type='submit' name='Submit' value='Submit'>
			      </td>
			    </tr>
			  </table>
			</form>";

		return 1;
		}


	}

?>